==> Net-HTTP2-nghttp2-0.003.tar.gz: Net-HTTP2-nghttp2-0.003/lib/Net/HTTP2/nghttp2.pm <==
package Net::HTTP2::nghttp2;

use strict;
use warnings;
use XSLoader;

our $VERSION = '0.003';

XSLoader::load('Net::HTTP2::nghttp2', $VERSION);

# Check if nghttp2 is available - must be after XSLoader
our $AVAILABLE = eval { _check_nghttp2_available() } ? 1 : 0;

sub available { return $AVAILABLE }

# Export constants
use Exporter 'import';
our @EXPORT_OK = qw(
    NGHTTP2_ERR_WOULDBLOCK
    NGHTTP2_ERR_CALLBACK_FAILURE
    NGHTTP2_ERR_DEFERRED
    NGHTTP2_FLAG_NONE
    NGHTTP2_FLAG_END_STREAM
    NGHTTP2_FLAG_END_HEADERS
    NGHTTP2_FLAG_ACK
    NGHTTP2_FLAG_PADDED
    NGHTTP2_FLAG_PRIORITY
    NGHTTP2_DATA_FLAG_NONE
    NGHTTP2_DATA_FLAG_EOF
    NGHTTP2_DATA_FLAG_NO_END_STREAM
    NGHTTP2_DATA_FLAG_NO_COPY
    NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS
    NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE
    NGHTTP2_SETTINGS_MAX_FRAME_SIZE
    NGHTTP2_SETTINGS_ENABLE_PUSH
    NGHTTP2_SETTINGS_ENABLE_CONNECT_PROTOCOL
);

our %EXPORT_TAGS = (
    all       => \@EXPORT_OK,
    errors    => [qw(NGHTTP2_ERR_WOULDBLOCK NGHTTP2_ERR_CALLBACK_FAILURE NGHTTP2_ERR_DEFERRED)],
    flags     => [qw(NGHTTP2_FLAG_NONE NGHTTP2_FLAG_END_STREAM NGHTTP2_FLAG_END_HEADERS
                     NGHTTP2_FLAG_ACK NGHTTP2_FLAG_PADDED NGHTTP2_FLAG_PRIORITY)],
    data      => [qw(NGHTTP2_DATA_FLAG_NONE NGHTTP2_DATA_FLAG_EOF
                     NGHTTP2_DATA_FLAG_NO_END_STREAM NGHTTP2_DATA_FLAG_NO_COPY)],
    settings  => [qw(NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE
                     NGHTTP2_SETTINGS_MAX_FRAME_SIZE NGHTTP2_SETTINGS_ENABLE_PUSH
                     NGHTTP2_SETTINGS_ENABLE_CONNECT_PROTOCOL)],
);

1;

__END__

=head1 NAME

Net::HTTP2::nghttp2 - Perl XS bindings for nghttp2 HTTP/2 library

=head1 SYNOPSIS

    use Net::HTTP2::nghttp2;
    use Net::HTTP2::nghttp2::Session;

    # Check availability
    die "nghttp2 not available" unless Net::HTTP2::nghttp2->available;

    # See EXAMPLES below for complete server and client examples

=head1 DESCRIPTION

This module provides Perl bindings for the nghttp2 C library, enabling
HTTP/2 protocol support in Perl applications.

nghttp2 is one of the most mature HTTP/2 implementations, used by curl,
Apache, and Firefox. It implements RFC 9113 (HTTP/2) and RFC 7541 (HPACK).

=head1 CLASS METHODS

=head2 available

    my $bool = Net::HTTP2::nghttp2->available;

Returns true if nghttp2 is available and properly linked.

=head1 CONSTANTS

=head2 Error Codes

=over 4

=item NGHTTP2_ERR_WOULDBLOCK

Operation would block (non-fatal).

=item NGHTTP2_ERR_CALLBACK_FAILURE

Callback returned an error.

=item NGHTTP2_ERR_DEFERRED

Data production deferred (for flow control).

=back

=head2 Frame Flags

=over 4

=item NGHTTP2_FLAG_END_STREAM

End of stream flag.

=item NGHTTP2_FLAG_END_HEADERS

End of headers flag.

=back

=head2 Settings Constants

=over 4

=item NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS

Maximum number of concurrent streams.

=item NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE

Initial flow control window size.

=item NGHTTP2_SETTINGS_MAX_FRAME_SIZE

Maximum frame size.

=item NGHTTP2_SETTINGS_ENABLE_PUSH

Enable/disable server push.

=item NGHTTP2_SETTINGS_ENABLE_CONNECT_PROTOCOL

Enable extended CONNECT protocol (RFC 8441). When enabled, allows
WebSocket bootstrapping over HTTP/2 using the extended CONNECT method
with a C<:protocol> pseudo-header.

=back

=head1 EXAMPLES

=head2 HTTP/2 Server (h2c - cleartext)

    use IO::Socket::INET;
    use Net::HTTP2::nghttp2;
    use Net::HTTP2::nghttp2::Session;

    my $server = IO::Socket::INET->new(
        LocalPort => 8080,
        Listen    => 128,
        ReuseAddr => 1,
    ) or die "Cannot create server: $!";

    while (my $client = $server->accept) {
        my $session = Net::HTTP2::nghttp2::Session->new_server(
            callbacks => {
                on_begin_headers => sub {
                    my ($stream_id) = @_;
                    # New request starting
                    return 0;
                },
                on_header => sub {
                    my ($stream_id, $name, $value) = @_;
                    # Received header: :method, :path, :scheme, etc.
                    return 0;
                },
                on_frame_recv => sub {
                    my ($frame) = @_;
                    # HEADERS frame with END_STREAM = complete request
                    if ($frame->{type} == 1 && ($frame->{flags} & 0x1)) {
                        $session->submit_response($frame->{stream_id},
                            status  => 200,
                            headers => [['content-type', 'text/plain']],
                            body    => "Hello, HTTP/2!\n",
                        );
                    }
                    return 0;
                },
                on_stream_close => sub {
                    my ($stream_id, $error_code) = @_;
                    return 0;
                },
            },
        );

        $session->send_connection_preface();

        # I/O loop
        while ($session->want_read || $session->want_write) {
            # Send pending data
            if (my $out = $session->mem_send) {
                $client->syswrite($out);
            }

            # Read incoming data
            my $buf;
            last unless $client->sysread($buf, 16384);
            $session->mem_recv($buf);
        }
        $client->close;
    }

=head2 HTTP/2 Client (h2 - TLS)

    use IO::Socket::SSL;
    use Net::HTTP2::nghttp2;
    use Net::HTTP2::nghttp2::Session;

    # Connect with ALPN to negotiate HTTP/2
    my $sock = IO::Socket::SSL->new(
        PeerHost           => 'nghttp2.org',
        PeerPort           => 443,
        SSL_alpn_protocols => ['h2'],
    ) or die "Connection failed: $!";

    die "ALPN failed" unless $sock->alpn_selected eq 'h2';

    my %response;
    my $session = Net::HTTP2::nghttp2::Session->new_client(
        callbacks => {
            on_header => sub {
                my ($stream_id, $name, $value) = @_;
                $response{headers}{$name} = $value;
                return 0;
            },
            on_data_chunk_recv => sub {
                my ($stream_id, $data) = @_;
                $response{body} .= $data;
                return 0;
            },
            on_stream_close => sub {
                my ($stream_id, $error_code) = @_;
                $response{done} = 1;
                return 0;
            },
        },
    );

    $session->send_connection_preface();

    # Submit GET request
    my $stream_id = $session->submit_request(
        method    => 'GET',
        path      => '/',
        scheme    => 'https',
        authority => 'nghttp2.org',
        headers   => [['user-agent', 'perl-nghttp2/0.001']],
    );

    # I/O loop
    while (!$response{done} && ($session->want_read || $session->want_write)) {
        if (my $out = $session->mem_send) {
            $sock->syswrite($out);
        }

        my $buf;
        last unless $sock->sysread($buf, 16384);
        $session->mem_recv($buf);
    }

    print "Status: $response{headers}{':status'}\n";
    print "Body: " . length($response{body}) . " bytes\n";

=head2 Streaming Response

    # In on_frame_recv callback, use a data provider for large responses:
    $session->submit_response($stream_id,
        status  => 200,
        headers => [['content-type', 'application/octet-stream']],
        body    => sub {
            my ($stream_id, $max_length) = @_;
            my $chunk = get_next_chunk($max_length);
            my $eof = is_last_chunk() ? 1 : 0;
            return ($chunk, $eof);
        },
    );

    # Return undef to defer, then call resume_stream() when data ready:
    body => sub {
        my ($stream_id, $max_length) = @_;
        return undef if !data_available();  # Defers
        return (get_data(), $eof);
    },

    # Later, when data becomes available:
    $session->resume_stream($stream_id);

=head1 CONFORMANCE TESTING

This module has been tested against h2spec (L<https://github.com/summerwind/h2spec>),
the HTTP/2 conformance testing tool.

=head2 h2spec Results

    146 tests, 137 passed, 1 skipped, 8 failed (94% pass rate)

=head2 Passing Test Categories

=over 4

=item * Starting HTTP/2 - Connection preface handling

=item * Streams and Multiplexing - Stream state management

=item * Frame Definitions - DATA, HEADERS, PRIORITY, RST_STREAM, SETTINGS, PING, GOAWAY, WINDOW_UPDATE, CONTINUATION

=item * HTTP Message Exchanges - GET, HEAD, POST requests with trailers

=item * HPACK - All header compression variants (indexed, literal, Huffman)

=item * Server Push - PUSH_PROMISE handling

=back

=head2 Known Limitations

The 8 failing tests are edge cases where nghttp2 intentionally chooses lenient
behavior over strict RFC compliance for better interoperability:

=over 4

=item * Invalid connection preface - nghttp2 sends SETTINGS before validating

=item * DATA/HEADERS on closed streams - Silently ignored rather than erroring

=item * Out-of-order stream identifiers - Accepted (lenient parsing)

=item * PRIORITY self-dependency - Ignored rather than treated as error

=item * PRIORITY on stream 0 - Silently ignored

=back

This lenient behavior is intentional in nghttp2 and matches the behavior of
production HTTP/2 implementations like curl, Apache, and nginx.

=head1 SEE ALSO

L<Alien::nghttp2> - Alien module for automatic nghttp2 installation

L<https://nghttp2.org/> - nghttp2 project homepage

L<https://datatracker.ietf.org/doc/html/rfc9113> - HTTP/2 RFC

L<https://datatracker.ietf.org/doc/html/rfc8441> - Bootstrapping WebSockets with HTTP/2

L<https://github.com/summerwind/h2spec> - HTTP/2 conformance testing tool

=head1 AUTHOR

Your Name <your@email.com>

=head1 LICENSE

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

==> Net-HTTP2-nghttp2-0.004.tar.gz: Net-HTTP2-nghttp2-0.004/lib/Net/HTTP2/nghttp2.pm <==
package Net::HTTP2::nghttp2;

use strict;
use warnings;
use XSLoader;

our $VERSION = '0.004';

XSLoader::load('Net::HTTP2::nghttp2', $VERSION);

# Check if nghttp2 is available - must be after XSLoader
our $AVAILABLE = eval { _check_nghttp2_available() } ? 1 : 0;

sub available { return $AVAILABLE }

# Export constants
use Exporter 'import';
our @EXPORT_OK = qw(
    NGHTTP2_ERR_WOULDBLOCK
    NGHTTP2_ERR_CALLBACK_FAILURE
    NGHTTP2_ERR_DEFERRED
    NGHTTP2_FLAG_NONE
    NGHTTP2_FLAG_END_STREAM
    NGHTTP2_FLAG_END_HEADERS
    NGHTTP2_FLAG_ACK
    NGHTTP2_FLAG_PADDED
    NGHTTP2_FLAG_PRIORITY
    NGHTTP2_DATA_FLAG_NONE
    NGHTTP2_DATA_FLAG_EOF
    NGHTTP2_DATA_FLAG_NO_END_STREAM
    NGHTTP2_DATA_FLAG_NO_COPY
    NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS
    NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE
    NGHTTP2_SETTINGS_MAX_FRAME_SIZE
    NGHTTP2_SETTINGS_ENABLE_PUSH
    NGHTTP2_SETTINGS_ENABLE_CONNECT_PROTOCOL
);

our %EXPORT_TAGS = (
    all       => \@EXPORT_OK,
    errors    => [qw(NGHTTP2_ERR_WOULDBLOCK NGHTTP2_ERR_CALLBACK_FAILURE NGHTTP2_ERR_DEFERRED)],
    flags     => [qw(NGHTTP2_FLAG_NONE NGHTTP2_FLAG_END_STREAM NGHTTP2_FLAG_END_HEADERS
                     NGHTTP2_FLAG_ACK NGHTTP2_FLAG_PADDED NGHTTP2_FLAG_PRIORITY)],
    data      => [qw(NGHTTP2_DATA_FLAG_NONE NGHTTP2_DATA_FLAG_EOF
                     NGHTTP2_DATA_FLAG_NO_END_STREAM NGHTTP2_DATA_FLAG_NO_COPY)],
    settings  => [qw(NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE
                     NGHTTP2_SETTINGS_MAX_FRAME_SIZE NGHTTP2_SETTINGS_ENABLE_PUSH
                     NGHTTP2_SETTINGS_ENABLE_CONNECT_PROTOCOL)],
);

1;

__END__

=head1 NAME

Net::HTTP2::nghttp2 - Perl XS bindings for nghttp2 HTTP/2 library

=head1 SYNOPSIS

    use Net::HTTP2::nghttp2;
    use Net::HTTP2::nghttp2::Session;

    # Check availability
    die "nghttp2 not available" unless Net::HTTP2::nghttp2->available;

    # See EXAMPLES below for complete server and client examples

=head1 DESCRIPTION

This module provides Perl bindings for the nghttp2 C library, enabling
HTTP/2 protocol support in Perl applications.

nghttp2 is one of the most mature HTTP/2 implementations, used by curl,
Apache, and Firefox. It implements RFC 9113 (HTTP/2) and RFC 7541 (HPACK).

=head1 CLASS METHODS

=head2 available

    my $bool = Net::HTTP2::nghttp2->available;

Returns true if nghttp2 is available and properly linked.

=head1 CONSTANTS

=head2 Error Codes

=over 4

=item NGHTTP2_ERR_WOULDBLOCK

Operation would block (non-fatal).

=item NGHTTP2_ERR_CALLBACK_FAILURE

Callback returned an error.

=item NGHTTP2_ERR_DEFERRED

Data production deferred (for flow control).

=back

=head2 Frame Flags

=over 4

=item NGHTTP2_FLAG_END_STREAM

End of stream flag.

=item NGHTTP2_FLAG_END_HEADERS

End of headers flag.

=back

=head2 Settings Constants

=over 4

=item NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS

Maximum number of concurrent streams.

=item NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE

Initial flow control window size.

=item NGHTTP2_SETTINGS_MAX_FRAME_SIZE

Maximum frame size.

=item NGHTTP2_SETTINGS_ENABLE_PUSH

Enable/disable server push.

=item NGHTTP2_SETTINGS_ENABLE_CONNECT_PROTOCOL

Enable extended CONNECT protocol (RFC 8441). When enabled, allows
WebSocket bootstrapping over HTTP/2 using the extended CONNECT method
with a C<:protocol> pseudo-header.

=back

=head1 EXAMPLES

=head2 HTTP/2 Server (h2c - cleartext)

    use IO::Socket::INET;
    use Net::HTTP2::nghttp2;
    use Net::HTTP2::nghttp2::Session;

    my $server = IO::Socket::INET->new(
        LocalPort => 8080,
        Listen    => 128,
        ReuseAddr => 1,
    ) or die "Cannot create server: $!";

    while (my $client = $server->accept) {
        my $session = Net::HTTP2::nghttp2::Session->new_server(
            callbacks => {
                on_begin_headers => sub {
                    my ($stream_id) = @_;
                    # New request starting
                    return 0;
                },
                on_header => sub {
                    my ($stream_id, $name, $value) = @_;
                    # Received header: :method, :path, :scheme, etc.
                    return 0;
                },
                on_frame_recv => sub {
                    my ($frame) = @_;
                    # HEADERS frame with END_STREAM = complete request
                    if ($frame->{type} == 1 && ($frame->{flags} & 0x1)) {
                        $session->submit_response($frame->{stream_id},
                            status  => 200,
                            headers => [['content-type', 'text/plain']],
                            body    => "Hello, HTTP/2!\n",
                        );
                    }
                    return 0;
                },
                on_stream_close => sub {
                    my ($stream_id, $error_code) = @_;
                    return 0;
                },
            },
        );

        $session->send_connection_preface();

        # I/O loop
        while ($session->want_read || $session->want_write) {
            # Send pending data
            if (my $out = $session->mem_send) {
                $client->syswrite($out);
            }

            # Read incoming data
            my $buf;
            last unless $client->sysread($buf, 16384);
            $session->mem_recv($buf);
        }
        $client->close;
    }

=head2 HTTP/2 Client (h2 - TLS)

    use IO::Socket::SSL;
    use Net::HTTP2::nghttp2;
    use Net::HTTP2::nghttp2::Session;

    # Connect with ALPN to negotiate HTTP/2
    my $sock = IO::Socket::SSL->new(
        PeerHost           => 'nghttp2.org',
        PeerPort           => 443,
        SSL_alpn_protocols => ['h2'],
    ) or die "Connection failed: $!";

    die "ALPN failed" unless $sock->alpn_selected eq 'h2';

    my %response;
    my $session = Net::HTTP2::nghttp2::Session->new_client(
        callbacks => {
            on_header => sub {
                my ($stream_id, $name, $value) = @_;
                $response{headers}{$name} = $value;
                return 0;
            },
            on_data_chunk_recv => sub {
                my ($stream_id, $data) = @_;
                $response{body} .= $data;
                return 0;
            },
            on_stream_close => sub {
                my ($stream_id, $error_code) = @_;
                $response{done} = 1;
                return 0;
            },
        },
    );

    $session->send_connection_preface();

    # Submit GET request
    my $stream_id = $session->submit_request(
        method    => 'GET',
        path      => '/',
        scheme    => 'https',
        authority => 'nghttp2.org',
        headers   => [['user-agent', 'perl-nghttp2/0.001']],
    );

    # I/O loop
    while (!$response{done} && ($session->want_read || $session->want_write)) {
        if (my $out = $session->mem_send) {
            $sock->syswrite($out);
        }

        my $buf;
        last unless $sock->sysread($buf, 16384);
        $session->mem_recv($buf);
    }

    print "Status: $response{headers}{':status'}\n";
    print "Body: " . length($response{body}) . " bytes\n";

=head2 Streaming Response

    # In on_frame_recv callback, use a data provider for large responses:
    $session->submit_response($stream_id,
        status  => 200,
        headers => [['content-type', 'application/octet-stream']],
        body    => sub {
            my ($stream_id, $max_length) = @_;
            my $chunk = get_next_chunk($max_length);
            my $eof = is_last_chunk() ? 1 : 0;
            return ($chunk, $eof);
        },
    );

    # Return undef to defer, then call resume_stream() when data ready:
    body => sub {
        my ($stream_id, $max_length) = @_;
        return undef if !data_available();  # Defers
        return (get_data(), $eof);
    },

    # Later, when data becomes available:
    $session->resume_stream($stream_id);

=head2 Streaming Request

Client-side requests also support streaming body callbacks, using the same
callback signature as streaming responses. This is essential for bidirectional
protocols like WebSocket over HTTP/2 (RFC 8441), where the client stream must
remain open for ongoing data exchange.

    my @send_queue;
    my $eof = 0;

    my $stream_id = $session->submit_request(
        method    => 'POST',
        path      => '/upload',
        scheme    => 'https',
        authority => 'example.com',
        body      => sub {
            my ($stream_id, $max_length) = @_;
            if (@send_queue) {
                my $chunk = shift @send_queue;
                return ($chunk, $eof);
            }
            return undef;  # Defer until data available
        },
    );

    # Later, queue data and resume:
    push @send_queue, $data;
    $session->resume_stream($stream_id);

=head2 WebSocket over HTTP/2 (RFC 8441)

RFC 8441 defines how to bootstrap WebSocket connections over HTTP/2 using the
extended CONNECT method. The server advertises support via
C<SETTINGS_ENABLE_CONNECT_PROTOCOL>, and the client sends an extended CONNECT
request with a C<:protocol> pseudo-header set to C<websocket>.

B<Server side> - enable the setting and detect extended CONNECT:

    $session->send_connection_preface(
        enable_connect_protocol => 1,
    );

    # In on_header callback, look for :protocol pseudo-header:
    on_header => sub {
        my ($stream_id, $name, $value) = @_;
        if ($name eq ':method' && $value eq 'CONNECT') {
            # Possible extended CONNECT
        }
        if ($name eq ':protocol' && $value eq 'websocket') {
            # This is a WebSocket upgrade request
        }
        return 0;
    },

    # Accept with 200 (not 101) and a streaming body for server-to-client:
    $session->submit_response($stream_id,
        status  => 200,
        headers => [['sec-websocket-protocol', $subprotocol]],
        body    => sub {
            my ($stream_id, $max_length) = @_;
            # Return WebSocket frames to send to client
            return undef;  # Defer until data ready
        },
    );

B<Client side> - send extended CONNECT with streaming body:

    my $stream_id = $session->submit_request(
        method    => 'CONNECT',
        path      => '/chat',
        scheme    => 'https',
        authority => 'example.com',
        headers   => [[':protocol', 'websocket']],
        body      => sub {
            my ($stream_id, $max_length) = @_;
            # Return WebSocket frames to send to server
            return undef;  # Defer until data ready
        },
    );

WebSocket frames (RFC 6455) are carried inside HTTP/2 DATA frames on the
stream. Both directions remain open until one side sends a DATA frame with
END_STREAM (by returning C<($data, 1)> from the callback).

=head1 CONFORMANCE TESTING

This module has been tested against h2spec (L<https://github.com/summerwind/h2spec>),
the HTTP/2 conformance testing tool.

=head2 h2spec Results

    146 tests, 137 passed, 1 skipped, 8 failed (94% pass rate)

=head2 Passing Test Categories

=over 4

=item * Starting HTTP/2 - Connection preface handling

=item * Streams and Multiplexing - Stream state management

=item * Frame Definitions - DATA, HEADERS, PRIORITY, RST_STREAM, SETTINGS, PING, GOAWAY, WINDOW_UPDATE, CONTINUATION

=item * HTTP Message Exchanges - GET, HEAD, POST requests with trailers

=item * HPACK - All header compression variants (indexed, literal, Huffman)

=item * Server Push - PUSH_PROMISE handling

=back

=head2 Known Limitations

The 8 failing tests are edge cases where nghttp2 intentionally chooses lenient
behavior over strict RFC compliance for better interoperability:

=over 4

=item * Invalid connection preface - nghttp2 sends SETTINGS before validating

=item * DATA/HEADERS on closed streams - Silently ignored rather than erroring

=item * Out-of-order stream identifiers - Accepted (lenient parsing)

=item * PRIORITY self-dependency - Ignored rather than treated as error

=item * PRIORITY on stream 0 - Silently ignored

=back

This lenient behavior is intentional in nghttp2 and matches the behavior of
production HTTP/2 implementations like curl, Apache, and nginx.

=head1 SEE ALSO

L<Alien::nghttp2> - Alien module for automatic nghttp2 installation

L<https://nghttp2.org/> - nghttp2 project homepage

L<https://datatracker.ietf.org/doc/html/rfc9113> - HTTP/2 RFC

L<https://datatracker.ietf.org/doc/html/rfc8441> - Bootstrapping WebSockets with HTTP/2

L<https://github.com/summerwind/h2spec> - HTTP/2 conformance testing tool

=head1 AUTHOR

John Napiorkowski <jjnapiork@cpan.org>

=head1 LICENSE

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
